model Schelling

global {
	float density <- 0.9;
	float preference <- 0.5;
	float distance <- 100.0;

	shape_file buildings_shapfile <- shape_file("../includes/buildings.shp");
	geometry shape <- envelope(buildings_shapfile);
	
	init {
		create house from: buildings_shapfile {
		    capacity <- int(1 + shape.area / 1000);
		}
		
		int total_capacity <- sum (house collect each.capacity);
		int nb_people <- int(total_capacity * density);
		
		create people number: nb_people {
		    my_house <- one_of (house where (each.capacity > 0)) ;
		 	ask my_house {
		           do go_in a_people:myself;
		    } 
		}
	}
	
	reflex end_simulation when: empty (people where not each.is_happy) {
		do pause;
	}	
}

species house skills:[moving] {
	int capacity;
	
	action go_in(people a_people) {
		capacity <- capacity - 1;
		a_people.location <- any_location_in(shape);
	}
	
	action go_out {
		capacity <- capacity + 1;
	}
	
	aspect default {
		draw shape color: #darkgrey border: #darkgrey;
	}
}

species people {
	house my_house;
	rgb color <- flip(0.5) ? #red : #yellow;
	bool is_happy;

	reflex move when: not is_happy {
		ask my_house {
		     do go_out;
		}
		my_house <- one_of (house where (each.capacity > 0));
		ask my_house {
		     do go_in a_people:myself;
		}
	}
	
	reflex compute_happiness {
		list<people> neigbours_pp <- people at_distance distance;
		int nb_neighbours <- length(neigbours_pp);
		int nb_neighbours_diff<- neigbours_pp count (each.color != color);
		is_happy <- (nb_neighbours = 0) or ((nb_neighbours_diff /nb_neighbours) < preference) ;
	}
	
	aspect cube {
		draw cube(20) color: color;
	}
}

experiment my_GUI_xp type:gui {
	parameter"density" var:density;
	parameter"preference" var:preference;
	parameter"distance" var:distance;

	output{		
		monitor "nb of happy people" value:people count each.is_happy;
		
		display map type: opengl {
			species house;
			species people aspect: cube;
		}
		
		display charts {
			chart "charts" type:series background:rgb(255,255,255){
				data "nb of happy people" style:line value:people count each.is_happy color:#green;
			}
		}
	}
}
